@@ -1,3 +1,4 @@
+from __future__ import unicode_literals
 from django.db import models
 from django.conf import settings
 from django.db.models import Q
@@ -6,34 +7,43 @@ from django.core.exceptions import ValidationError
 
 from django.utils import timezone
 from django.utils.translation import ugettext_lazy as _
+from django.utils.encoding import python_2_unicode_compatible
 
-from friendship.exceptions import AlreadyExistsError
-from friendship.signals import friendship_request_created, \
-    friendship_request_rejected, friendship_request_canceled, \
-    friendship_request_viewed, friendship_request_accepted, \
-    friendship_removed, follower_created, following_created, follower_removed,\
-    following_removed
+from friendship.exceptions import AlreadyExistsError, AlreadyFriendsError
+from friendship.signals import (
+    friendship_request_created, friendship_request_rejected,
+    friendship_request_canceled,
+    friendship_request_viewed, friendship_request_accepted,
+    friendship_removed, follower_created, follower_removed,
+    followee_created, followee_removed, following_created, following_removed, block_created,block_removed
+)
 
 AUTH_USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')
 
 CACHE_TYPES = {
-    'friends': 'f-%d',
-    'followers': 'fo-%d',
-    'following': 'fl-%d',
-    'requests': 'fr-%d',
-    'sent_requests': 'sfr-%d',
-    'unread_requests': 'fru-%d',
-    'unread_request_count': 'fruc-%d',
-    'read_requests': 'frr-%d',
-    'rejected_requests': 'frj-%d',
-    'unrejected_requests': 'frur-%d',
-    'unrejected_request_count': 'frurc-%d',
+    'friends': 'f-%s',
+    'followers': 'fo-%s',
+    'following': 'fl-%s',
+    'blocks': 'b-%s',
+    'blocked': 'bo-%s',
+    'blocking': 'bd-%s',
+    'requests': 'fr-%s',
+    'sent_requests': 'sfr-%s',
+    'unread_requests': 'fru-%s',
+    'unread_request_count': 'fruc-%s',
+    'read_requests': 'frr-%s',
+    'rejected_requests': 'frj-%s',
+    'unrejected_requests': 'frur-%s',
+    'unrejected_request_count': 'frurc-%s',
 }
 
 BUST_CACHES = {
     'friends': ['friends'],
     'followers': ['followers'],
+    'blocks': ['blocks'],
+    'blocked': ['blocked'],
     'following': ['following'],
+    'blocking': ['blocking'],
     'requests': [
         'requests',
         'unread_requests',
@@ -63,10 +73,11 @@ def bust_cache(type, user_pk):
     cache.delete_many(keys)
 
 
+@python_2_unicode_compatible
 class FriendshipRequest(models.Model):
     """ Model to represent friendship requests """
-    from_user = models.ForeignKey(AUTH_USER_MODEL, related_name='friendship_requests_sent')
-    to_user = models.ForeignKey(AUTH_USER_MODEL, related_name='friendship_requests_received')
+    from_user = models.ForeignKey(AUTH_USER_MODEL, on_delete=models.CASCADE, related_name='friendship_requests_sent')
+    to_user = models.ForeignKey(AUTH_USER_MODEL, on_delete=models.CASCADE, related_name='friendship_requests_received')
 
     message = models.TextField(_('Message'), blank=True)
 
@@ -79,8 +90,8 @@ class FriendshipRequest(models.Model):
         verbose_name_plural = _('Friendship Requests')
         unique_together = ('from_user', 'to_user')
 
-    def __unicode__(self):
-        return "User #%d friendship requested #%d" % (self.from_user_id, self.to_user_id)
+    def __str__(self):
+        return "%s" % self.from_user_id
 
     def accept(self):
         """ Accept this friendship request """
@@ -271,25 +282,45 @@ class FriendshipManager(models.Manager):
         if from_user == to_user:
             raise ValidationError("Users cannot be friends with themselves")
 
+        if self.are_friends(from_user, to_user):
+            raise AlreadyFriendsError("Users are already friends")
+
+        if self.can_request_send(from_user, to_user):
+            raise AlreadyExistsError("Friendship already requested")
+
         if message is None:
             message = ''
 
         request, created = FriendshipRequest.objects.get_or_create(
             from_user=from_user,
             to_user=to_user,
-            message=message,
         )
 
         if created is False:
             raise AlreadyExistsError("Friendship already requested")
 
+        if message:
+            request.message = message
+            request.save()
+
         bust_cache('requests', to_user.pk)
         bust_cache('sent_requests', from_user.pk)
         friendship_request_created.send(sender=request)
 
         return request
 
-    def remove_friend(self, to_user, from_user):
+    def can_request_send(self, from_user, to_user):
+        """ Checks if a request was sent """
+        if from_user == to_user or FriendshipRequest.objects.filter(
+            from_user=from_user,
+            to_user=to_user,
+            ).exists() is False:
+            return False
+        else: 
+            return True
+
+
+    def remove_friend(self, from_user, to_user):
         """ Destroy a friendship relationship """
         try:
             qs = Friend.objects.filter(
@@ -328,10 +359,11 @@ class FriendshipManager(models.Manager):
                 return False
 
 
+@python_2_unicode_compatible
 class Friend(models.Model):
     """ Model to represent Friendships """
-    to_user = models.ForeignKey(AUTH_USER_MODEL, related_name='friends')
-    from_user = models.ForeignKey(AUTH_USER_MODEL, related_name='_unused_friend_relation')
+    to_user = models.ForeignKey(AUTH_USER_MODEL, models.CASCADE, related_name='friends')
+    from_user = models.ForeignKey(AUTH_USER_MODEL, models.CASCADE, related_name='_unused_friend_relation')
     created = models.DateTimeField(default=timezone.now)
 
     objects = FriendshipManager()
@@ -341,8 +373,8 @@ class Friend(models.Model):
         verbose_name_plural = _('Friends')
         unique_together = ('from_user', 'to_user')
 
-    def __unicode__(self):
-        return "User #%d is friends with #%d" % (self.to_user_id, self.from_user_id)
+    def __str__(self):
+        return "User #%s is friends with #%s" % (self.to_user_id, self.from_user_id)
 
     def save(self, *args, **kwargs):
         # Ensure users can't be friends with themselves
@@ -389,7 +421,8 @@ class FollowingManager(models.Manager):
             raise AlreadyExistsError("User '%s' already follows '%s'" % (follower, followee))
 
         follower_created.send(sender=self, follower=follower)
-        following_created.send(sender=self, follow=followee)
+        followee_created.send(sender=self, followee=followee)
+        following_created.send(sender=self, following=relation)
 
         bust_cache('followers', followee.pk)
         bust_cache('following', follower.pk)
@@ -401,7 +434,8 @@ class FollowingManager(models.Manager):
         try:
             rel = Follow.objects.get(follower=follower, followee=followee)
             follower_removed.send(sender=rel, follower=rel.follower)
-            following_removed.send(sender=rel, following=rel.followee)
+            followee_removed.send(sender=rel, followee=rel.followee)
+            following_removed.send(sender=rel, following=rel)
             rel.delete()
             bust_cache('followers', followee.pk)
             bust_cache('following', follower.pk)
@@ -426,10 +460,11 @@ class FollowingManager(models.Manager):
                 return False
 
 
+@python_2_unicode_compatible
 class Follow(models.Model):
     """ Model to represent Following relationships """
-    follower = models.ForeignKey(AUTH_USER_MODEL, related_name='following')
-    followee = models.ForeignKey(AUTH_USER_MODEL, related_name='followers')
+    follower = models.ForeignKey(AUTH_USER_MODEL, models.CASCADE, related_name='following')
+    followee = models.ForeignKey(AUTH_USER_MODEL, models.CASCADE, related_name='followers')
     created = models.DateTimeField(default=timezone.now)
 
     objects = FollowingManager()
@@ -439,11 +474,112 @@ class Follow(models.Model):
         verbose_name_plural = _('Following Relationships')
         unique_together = ('follower', 'followee')
 
-    def __unicode__(self):
-        return "User #%d follows #%d" % (self.follower_id, self.followee_id)
+    def __str__(self):
+        return "User #%s follows #%s" % (self.follower_id, self.followee_id)
 
     def save(self, *args, **kwargs):
         # Ensure users can't be friends with themselves
         if self.follower == self.followee:
             raise ValidationError("Users cannot follow themselves.")
         super(Follow, self).save(*args, **kwargs)
+
+
+class BlockManager(models.Manager):
+    """ Following manager """
+
+    def blocked(self, user):
+        """ Return a list of all blocks """
+        key = cache_key('blocked', user.pk)
+        blocked = cache.get(key)
+
+        if blocked is None:
+            qs = Block.objects.filter(blocked=user).all()
+            blocked = [u.blocked for u in qs]
+            cache.set(key, blocked)
+
+        return blocked
+
+    def blocking(self, user):
+        """ Return a list of all users the given user blocks """
+        key = cache_key('blocking', user.pk)
+        blocking = cache.get(key)
+
+        if blocking is None:
+            qs = Block.objects.filter(blocker=user).all()
+            blocking = [u.blocked for u in qs]
+            cache.set(key, blocking)
+
+        return blocking
+
+    def add_block(self, blocker, blocked):
+        """ Create 'follower' follows 'followee' relationship """
+        if blocker == blocked:
+            raise ValidationError("Users cannot block themselves")
+
+        relation, created = Block.objects.get_or_create(blocker=blocker, blocked=blocked)
+
+        if created is False:
+            raise AlreadyExistsError("User '%s' already blocks '%s'" % (blocker, blocked))
+
+        block_created.send(sender=self, blocker=blocker)
+        block_created.send(sender=self, blocked=blocked)
+        block_created.send(sender=self, blocking=relation)
+
+        bust_cache('blocked', blocked.pk)
+        bust_cache('blocking', blocker.pk)
+
+        return relation
+
+    def remove_block(self, blocker, blocked):
+        """ Remove 'blocker' blocks 'blocked' relationship """
+        try:
+            rel = Block.objects.get(blocker=blocker, blocked=blocked)
+            block_removed.send(sender=rel, blocker=rel.blocker)
+            block_removed.send(sender=rel, blocked=rel.blocked)
+            block_removed.send(sender=rel, blocking=rel)
+            rel.delete()
+            bust_cache('blocked', blocked.pk)
+            bust_cache('blocking', blocker.pk)
+            return True
+        except Follow.DoesNotExist:
+            return False
+
+    def is_blocked(self, user1, user2):
+        """ Are these two users blocked? """
+        block1 = cache.get(cache_key('blocks', user1.pk))
+        block2 = cache.get(cache_key('blocks', user2.pk))
+        if block1 and user2 in block1:
+            return True
+        elif block2 and user1 in block2:
+            return True
+        else:
+            try:
+                Block.objects.get(blocker=user1, blocked=user2)
+                return True
+            except Block.DoesNotExist:
+                return False
+
+
+
+@python_2_unicode_compatible
+class Block(models.Model):
+    """ Model to represent Following relationships """
+    blocker = models.ForeignKey(AUTH_USER_MODEL, on_delete=models.CASCADE, related_name='blocking')
+    blocked = models.ForeignKey(AUTH_USER_MODEL, on_delete=models.CASCADE, related_name='blockees')
+    created = models.DateTimeField(default=timezone.now)
+
+    objects = BlockManager()
+
+    class Meta:
+        verbose_name = _('Blocker Relationship')
+        verbose_name_plural = _('Blocked Relationships')
+        unique_together = ('blocker', 'blocked')
+
+    def __str__(self):
+        return "User #%s blocks #%s" % (self.blocker_id, self.blocked_id)
+
+    def save(self, *args, **kwargs):
+        # Ensure users can't be friends with themselves
+        if self.blocker == self.blocked:
+            raise ValidationError("Users cannot block themselves.")
+        super( Block, self).save(*args, **kwargs)
