@@ -3,11 +3,11 @@ from django.contrib.auth.models import User
 from django.core.cache import cache
 from django.core.exceptions import ValidationError
 from django.db import IntegrityError
-from django.core.urlresolvers import reverse
+from django.urls import reverse
 from django.test import TestCase
 
-from friendship.exceptions import AlreadyExistsError
-from friendship.models import Friend, Follow, FriendshipRequest
+from friendship.exceptions import AlreadyExistsError, AlreadyFriendsError
+from friendship.models import Friend, Follow, FriendshipRequest, Block
 
 
 TEST_TEMPLATES = os.path.join(os.path.dirname(__file__), 'templates')
@@ -73,6 +73,11 @@ class FriendshipModelTests(BaseTestCase):
         ### Bob wants to be friends with Steve
         req1 = Friend.objects.add_friend(self.user_bob, self.user_steve)
 
+        # Ensure that the request can be sent
+        self.assertFalse(Friend.objects.can_request_send(self.user_bob, self.user_bob))
+        self.assertTrue(Friend.objects.can_request_send(self.user_bob, self.user_steve))
+        
+
         # Ensure neither have friends already
         self.assertEqual(Friend.objects.friends(self.user_bob), [])
         self.assertEqual(Friend.objects.friends(self.user_steve), [])
@@ -131,8 +136,6 @@ class FriendshipModelTests(BaseTestCase):
         req3.reject()
 
         # Duplicated requests raise a more specific subclass of IntegrityError.
-        with self.assertRaises(IntegrityError):
-            Friend.objects.add_friend(self.user_susan, self.user_amy)
         with self.assertRaises(AlreadyExistsError):
             Friend.objects.add_friend(self.user_susan, self.user_amy)
 
@@ -158,6 +161,14 @@ class FriendshipModelTests(BaseTestCase):
         with self.assertRaises(ValidationError):
             Friend.objects.create(to_user=self.user_bob, from_user=self.user_bob)
 
+    def test_already_friends_with_request(self):
+        # Make Bob and Steve friends
+        req = Friend.objects.add_friend(self.user_bob, self.user_steve)
+        req.accept()
+
+        with self.assertRaises(AlreadyFriendsError):
+            req2 = Friend.objects.add_friend(self.user_bob, self.user_steve)
+
     def test_multiple_friendship_requests(self):
         """ Ensure multiple friendship requests are handled properly """
         ### Bob wants to be friends with Steve
@@ -187,6 +198,13 @@ class FriendshipModelTests(BaseTestCase):
         self.assertEqual(FriendshipRequest.objects.filter(from_user=self.user_steve).count(), 0)
         self.assertEqual(FriendshipRequest.objects.filter(to_user=self.user_bob).count(), 0)
 
+    def test_multiple_calls_add_friend(self):
+        """ Ensure multiple calls with same friends, but different message works as expected """
+        req1 = Friend.objects.add_friend(self.user_bob, self.user_steve, message='Testing')
+
+        with self.assertRaises(AlreadyExistsError):
+            req2 = Friend.objects.add_friend(self.user_bob, self.user_steve, message='Foo Bar')
+
     def test_following(self):
         # Bob follows Steve
         req1 = Follow.objects.add_follower(self.user_bob, self.user_steve)
@@ -217,6 +235,31 @@ class FriendshipModelTests(BaseTestCase):
         with self.assertRaises(ValidationError):
             Follow.objects.create(follower=self.user_bob, followee=self.user_bob)
 
+    def test_blocking(self):
+        # Bob blocks Steve
+        req1 = Block.objects.add_block(self.user_bob, self.user_steve)
+        self.assertEqual(len(Block.objects.blocking(self.user_bob)), 1)
+        self.assertEqual(len(Block.objects.blocked(self.user_steve)), 1)
+        self.assertEqual(Block.objects.is_blocked(self.user_bob, self.user_steve), True)
+
+        # Duplicated requests raise a more specific subclass of IntegrityError.
+        with self.assertRaises(IntegrityError):
+            Block.objects.add_block(self.user_bob, self.user_steve)
+        with self.assertRaises(AlreadyExistsError):
+            Block.objects.add_block(self.user_bob, self.user_steve)
+
+        # Remove the relationship
+        self.assertTrue(Block.objects.remove_block(self.user_bob, self.user_steve))
+        self.assertEqual(len(Block.objects.blocking(self.user_steve)), 0)
+        self.assertEqual(len(Block.objects.blocked(self.user_bob)), 0)
+
+        # Ensure we canot block ourselves
+        with self.assertRaises(ValidationError):
+            Block.objects.add_block(self.user_bob, self.user_bob)
+
+        with self.assertRaises(ValidationError):
+            Block.objects.create(blocker=self.user_bob, blocked=self.user_bob)
+
 
 class FriendshipViewTests(BaseTestCase):
 
@@ -397,7 +440,6 @@ class FriendshipViewTests(BaseTestCase):
             redirect_url = reverse('friendship_request_list')
             self.assertTrue(redirect_url in response['Location'])
 
-
     def test_friendship_requests_detail(self):
         url = reverse('friendship_requests_detail', kwargs={'friendship_request_id': self.friendship_request.pk})
 
@@ -474,3 +516,69 @@ class FriendshipViewTests(BaseTestCase):
             self.assertResponse302(response)
             redirect_url = reverse('friendship_following', kwargs={'username': self.user_bob.username})
             self.assertTrue(redirect_url in response['Location'])
+
+    def test_friendship_blockers(self):
+        url = reverse('friendship_blockers', kwargs={'username': 'bob'})
+
+        # test that the view requires authentication to access it
+        response = self.client.get(url)
+        self.assertResponse200(response)
+
+        with self.settings(FRIENDSHIP_CONTEXT_OBJECT_NAME='object', TEMPLATE_DIRS=(TEST_TEMPLATES,)):
+            response = self.client.get(url)
+            self.assertResponse200(response)
+            self.assertTrue('object' in response.context)
+
+    def test_friendship_blocking(self):
+        url = reverse('friendship_blocking', kwargs={'username': 'bob'})
+
+        # test that the view requires authentication to access it
+        response = self.client.get(url)
+        self.assertResponse200(response)
+
+        with self.settings(FRIENDSHIP_CONTEXT_OBJECT_NAME='object', TEMPLATE_DIRS=(TEST_TEMPLATES,)):
+            response = self.client.get(url)
+            self.assertResponse200(response)
+            self.assertTrue('object' in response.context)
+
+    def test_block_add(self):
+        url = reverse('block_add', kwargs={'blocked_username': self.user_amy.username})
+
+        # test that the view requires authentication to access it
+        response = self.client.get(url)
+        self.assertResponse302(response)
+
+        with self.login(self.user_bob.username, self.user_pw):
+            response = self.client.get(url)
+            self.assertResponse200(response)
+
+            # on POST accept the friendship request and redirect to the
+            # friendship_following view
+            response = self.client.post(url)
+            self.assertResponse302(response)
+            redirect_url = reverse('friendship_blocking', kwargs={'username': self.user_bob.username})
+            self.assertTrue(redirect_url in response['Location'])
+
+            response = self.client.post(url)
+            self.assertResponse200(response)
+            self.assertTrue('errors' in response.context)
+            self.assertEqual(response.context['errors'], ["User 'bob' already blocks 'amy'"])
+
+    def test_block_remove(self):
+        # create a follow relationship so we can test removing a block
+        block = Block.objects.add_block(self.user_bob, self.user_amy)
+
+        url = reverse('block_remove', kwargs={'blocked_username': self.user_amy.username})
+
+        # test that the view requires authentication to access it
+        response = self.client.get(url)
+        self.assertResponse302(response)
+
+        with self.login(self.user_bob.username, self.user_pw):
+            response = self.client.get(url)
+            self.assertResponse200(response)
+
+            response = self.client.post(url)
+            self.assertResponse302(response)
+            redirect_url = reverse('friendship_blocking', kwargs={'username': self.user_bob.username})
+            self.assertTrue(redirect_url in response['Location'])
\ No newline at end of file
