@@ -1,13 +1,13 @@
 try:
-    from django.conf.urls import url, patterns
+    from django.conf.urls import url
 except ImportError:
-    from django.conf.urls.defaults import url, patterns
+    from django.conf.urls.defaults import url
 from friendship.views import view_friends, friendship_add_friend, friendship_accept, \
-        friendship_reject, friendship_cancel, friendship_request_list, \
-        friendship_request_list_rejected, friendship_requests_detail, followers,\
-        following, follower_add, follower_remove, all_users
+    friendship_reject, friendship_cancel, friendship_request_list, \
+    friendship_request_list_rejected, friendship_requests_detail, followers,\
+    following, follower_add, follower_remove, all_users,block_add,block_remove,blockers,blocking
 
-urlpatterns = patterns('',
+urlpatterns = [
     url(
         regex=r'^users/$',
         view=all_users,
@@ -73,4 +73,24 @@ urlpatterns = patterns('',
         view=follower_remove,
         name='follower_remove',
     ),
-)
+    url(
+        regex=r'^blockers/(?P<username>[\w-]+)/$',
+        view=blockers,
+        name='friendship_blockers',
+    ),
+    url(
+        regex=r'^blocking/(?P<username>[\w-]+)/$',
+        view=blocking,
+        name='friendship_blocking',
+    ),
+    url(
+        regex=r'^block/add/(?P<blocked_username>[\w-]+)/$',
+        view=block_add,
+        name='block_add',
+    ),
+    url(
+        regex=r'^block/remove/(?P<blocked_username>[\w-]+)/$',
+        view=block_remove,
+        name='block_remove',
+    ),
+]
